entity = 
{

	x=0.000,
	y=0.000,
	current_sprite=1,
	current_collision=1,				--says what set of collision rects the entity is currently on
	class = {},					--says what class of object this entity is: REQUIRED; a table
	id=nil
	
}
entity.__index = entity



function entity:new(variables)

	variables=variables or {}
	variables._oldx=variables.x or 0.000
	variables._oldy=variables.y or 0.000
	variables.class=variables.class or {}
	
	local self=setmetatable(variables,self)
	self:runScript("create")
	
	return self

end



function entity:move(relx, rely)

	--self.x+relx*dt									--*dt to go off game speed

end



function entity:drawSelf(display_x, display_y, scale_factor)

	scale_factor=scale_factor or 1
	if self.class.sprite_quads[self.currentsprite]~=nil then
	love.graphics.draw(self.class.spritesheet, self.class.sprite_quads[self.currentsprite], display_x, display_y, 0, scale_factor, scale_factor, 0, 0, 0, 0) end
	--else print("Error: sprite_quads[" .. self.currentsprite .. "] is null") end

end



function entity:getRelRect()

	return createRectangle(self.x,self.y,self.class._boundingBox_collision[self.current_collision].w,self.class._boundingBox_collision[self.current_collision].h)

end



function entity:runScript(script_name,...)

	if script_name=="draw" then love.graphics.setColor(255,255,255,255)
	elseif script_name=="destroy" then return nil end									--won't run destroy script (will crash the game)
	
	if pcall(self.class.script[script_name],self,...) then return true
	else return false end

end



function entity:checkCollision(collided_obj)

	if checkRectangleCollision(self:getRelRect(),collided_obj:getRelRect()) then
	--check if it is possible if there could be a collision; if so...

		local self_rect={}			--create relative rectangles
		local other_rect={}

		for i=1,table.getn(self.class.collision_rects[self.current_collision].rectangles),1 do
		
			self_rect={
				
				x=self.x+self.class.collision_rects[self.current_collision].rectangles[i].x,			--values won't change if there's no movement
				y=self.y+self.class.collision_rects[self.current_collision].rectangles[i].y,
				w=self.class.collision_rects[self.current_collision].rectangles[i].w,
				h=self.class.collision_rects[self.current_collision].rectangles[i].h
				
			}
			for v=1,table.getn(collided_obj.class.collision_rects[collided_obj.current_collision].rectangles) do
			
				other_rect={
				
					x=collided_obj.x+collided_obj.class.collision_rects[collided_obj.current_collision].rectangles[i].x,			--values won't change if there's no movement
					y=collided_obj.y+collided_obj.class.collision_rects[collided_obj.current_collision].rectangles[i].y,
					w=collided_obj.class.collision_rects[collided_obj.current_collision].rectangles[i].w,
					h=collided_obj.class.collision_rects[collided_obj.current_collision].rectangles[i].h
				
				}
				--check all rectangles for a collision
				if checkRectangleCollision(self_rect,other_rect) then
				
					return true		--there is a collision
				
				end
			
			end
	
		end
		
	end
	return false 			--there is no collision

end



function entity:fixCollision(collided_obj,axis)			--handles the collision for self and the other entity

	local self_rect=self:getRelRect()

	if collided_obj._notEntity then						--check if the collision is not from an entity; if so...
		
		if axis=='X' then											--fix collision based on current axis
			
			if self:getDirection('X')=="left" then
				
				self.x=other_rect.x+other_rect.w
				
			elseif self.getDirection('X')=="right" then
				
				self.x=other_rect.x-self_rect.w
				
			end
			
			self._oldx=self.x
				
		elseif axis=='Y' then
			
			if self:getDirection('Y')=="down" then				--self moved down
				
				self.y=other_rect.y+other_rect.h
				
			elseif self:getDirection('Y')=="up" then				--self moved up
				
				self.y=other_rect.y-self_rect.h
				
			end
			
			self._oldy=self.y
			
		end
		
		
	else								--the collision is from an entity

		local col_ty=self:getCollisionType(collided_obj,axis)
		local other_rect=collided_obj:getRelRect()
		if axis=='X' then										--fix collision based on the current axis

			if col_ty=="LR" then
				
				local self_oldx=self.x
				local other_oldx=collided_obj.x
				local self_speed=self._oldx-self._x			--get the speed
				local other_speed=collided_obj.x-collided_obj._oldx
				local collided_pixels=collided_obj.x+other_rect.w-self.x
				
				--divide each speed the two speeds added up(gets a fraction that when both added =1); then multiply by the amount of collided pixels
				self.x=self.x+((self_speed/(self_speed+other_speed))*collided_pixels)
				self._oldx=self_oldx
				collided_obj.x=collided_obj.x-((other_speed/(self_speed+other_speed))*collided_pixels)
				collided_obj._oldx=other_oldx
				
			elseif col_ty=="LL" then
				
				if self.x<collided_obj.x then				--determine which object is in front and push back the object that is behind
					
					collided_obj._oldx=collided_obj.x
					collided_obj.x=self.x+self_rect.w							--self is in front
					
				else								--collided_obj is in front of self
					
					self._oldx=self.x
					self.x=collided_obj.x+other_rect.w
					
				end
				
			elseif col_ty=="RR" then
			
				if self.x<collided_obj.x then				--determine which object is in front and push back the object that is behind
					
					self._oldx=self.x
					self.x=collided_obj.x-self_rect.w						--self is in back
					
				else								--collided_obj is behind self
					
					collided_obj._oldx=collided_obj.x
					collided_obj.x=self.x-other_rect.w
					
				end
				
			elseif col_ty=="RL" then

				local self_oldx=self.x
				local other_oldx=collided_obj.x
				local self_speed=self.x-self._oldx			--get the speed
				local other_speed=collided_obj._oldx-collided_obj.x
				local collided_pixels=self.x+self_rect.w-collided_obj.x
				
				--divide each speed the two speeds added up(gets a fraction that when both added =1); then multiply by the amount of collided pixels
				self.x=self.x-((self_speed/(self_speed+other_speed))*collided_pixels)
				self._oldx=self_oldx
				collided_obj.x=collided_obj.x+((other_speed/(self_speed+other_speed))*collided_pixels)
				collided_obj._oldx=other_oldx
				
			elseif col_ty=="NR" then
			
				collided_obj._oldx=collided_obj.x
				collided_obj.x=self.x-other_rect.w
			
			elseif col_ty=="NL" then
			
				collided_obj._oldx=collided_obj.x
				collided_obj.x=self.x+self_rect.w
				
			elseif col_ty=="RN" then
			
				self._oldx=self.x
				self.x=collided_obj.x-self_rect.w
			
			elseif col_ty=="LN" then
			
				self._oldx=self.x
				self.x=collided_obj.x+other_rect.w
				
			else					--collision is "NN" {ONLY POSSIBLE WITH MORE THAN TWO OBJECTS}
			
				print("NN COLLISION CAN HAPPEN!        AXIS: X")
			
			end
			self._oldx=self.x
			collided_obj._oldx=collided_obj.x
				
		elseif axis=='Y' then			--current axis is the y axis

		
		end
		
	end

end



function entity:getCollisionType(collided_obj,axis)			--returns self direction then collided_obj direction; can be l, r, u, d, n

	if collided_obj._notEntity then return nil end			--make sure the collided_obj is actually an entity
	
	if axis=='X' then
	
		if self:getDirection('X')=="left" then
		
			if collided_obj:getDirection('X')=="left" then return "LL"				--returns that self and other are going the same way
			elseif collided_obj:getDirection('X')=="right" then return "LR"		--collision for self goes left and other goes right
			else return "LN" end																--self moves left and right moves none
			
		elseif self:getDirection('X')=="right" then
		
			if collided_obj:getDirection('X')=="left" then return "RL"
			elseif collided_obj:getDirection('X')=="right" then return "RR"
			else return "RN" end
		
		else																									--no direction
		
			if collided_obj:getDirection('X')=="left" then return "NL"
			elseif collided_obj:getDirection('X')=="right" then return "NR"
			else return "NN" end
		
		end
	
	elseif axis=='Y' then
	
		if self:getDirection('Y')=="down" then
		
			if collided_obj:getDirection('Y')=="down" then return "DD"			--returns that self and other are going the same way
			elseif collided_obj:getDirection('Y')=="up" then return "DU"			--collision for self goes left and other goes right
			else return "DN" end																--self moves left and right moves none
			
		elseif self:getDirection('Y')=="up" then
		
			if collided_obj:getDirection('Y')=="" then return "RL"
			elseif collided_obj:getDirection('Y')=="right" then return "RR"
			else return "RN" end
		
		else																									--no direction
		
			if collided_obj:getDirection('X')=="left" then return "NL"
			elseif collided_obj:getDirection('X')=="right" then return "NR"
			else return "NN" end
		
		end
	
	end

end



function entity:getDirection(axis)			--gets the direction self moved

	if axis=='X' then

		if self.x<self._oldx then return "left"
		elseif self.x>self._oldx then return "right"
		else return "none" end
		
	elseif axis=='Y' then

		if self.y<self._oldy then return "up"
		elseif self.y>self._oldy then return "down"
		else return "none" end
	
	end

end



function entity:getMovementRect()		--doesn't work properly if x and y are moved at the same time

	--local xdir=self:getDirection('X')
	--local ydir=self:getDirection('Y')
	local retrect={
	
		x=self.x+self.class.collision_rects[self.current_collision].rectangles[i].x,			--values won't change if there's no movement
		y=self.y+self.class.collision_rects[self.current_collision].rectangles[i].y,
		w=self.class.collision_rects[self.current_collision].rectangles[i].w,
		h=self.class.collision_rects[self.current_collision].rectangles[i].h
	
	}
	--[[if xdir=="left" then					--adjusts the rectangle based on the direction moved
	
		retrect.x=self.x
		retrect.w=self._oldx-self.x
	
	elseif xdir=="right" then
	
		retrect.x=self._oldx
		retrect.w=self.x-self._oldx
	
	end
	
	
	if ydir=="up" then
	
	elseif ydir=="down" then
	
	end--]]
	
	return retrect

end



function entity:destroy()

	if pcall(self.class.script.destroy,self,"entity") then return true		--run class destroy script
	else return false end

	self.x=nil
	self.y=nil
	self.currentsprite=nil
	self.class=nil
	self.id=nil
	
	collectgarbage()				--remove vars from memory

end