ASCII_font =
{

	fontsheet=nil,			--image file
	rows=16,
	columns=16,
	char_spacing=0,			--how much to space each character away from each other
	box_borderw=0,		--how much of a box border for the x axis
	box_borderh=bow_borderw,			--for the y axis
	spacechar_spacing=4,			--how many pixels the space character is worth
	vspacing=10,					--tells how many pixels to skip down when entering a new line
	tab_spacing=4						--tells how many spaces the tab character spacing is
	--asciistart_right=nil					--tells if the  font sheet starts its table to the right, only used if true
	--asciistart_bottom=nil				--tells if the font sheet starts its table at the bottom, only used if true
	--upDown_sheet=nil				--tells if the font sheet goes from top to bottom and then changes the x-axis
	
}
ASCII_font.__index = ASCII_font


function ASCII_font:new(variables)

	local variables=variables or {}
	
	variables._charpositions = {}			--love2d quad used for drawing and telling where the characters are positioned within the font sheet
	variables._charheights = {}			--tells how much to move each character down (so that they are evenly aligned)
	variables._charwidths = {}			--used for proper spacing when drawing text
	
	local self=setmetatable(variables,self)
	return self

end



function ASCII_font:build()

	if type(fontsheet)~=nil  then
	
		if self.box_borderw==nil then self.box_borderw=0 end		--if no box is set then make use 0 (so none of the box math is done)
		if self.box_borderh==nil then self.box_borderh=0 end
		
		local char_width=(self.fontsheet:getWidth()-(self.box_borderw*(self.rows+1)))/self.rows			--store the width and height so it knows when to skip to the next character
		local char_height=(self.fontsheet:getHeight()-(self.box_borderh*(self.columns+1)))/self.columns
		local start_row=0					--row and column that started being used
		local start_column=0	
		local add_row=0					--tells whether to add -1, +1, or 0 to the row or column
		local add_column=0
		local current_row=0
		local current_column=0
		local fontsheet_data=self.fontsheet:getData()			--imageData of fontsheet, so the function can use getPixel
		local cc_counter=0		--used for shifting to the next row/column
		
		-- Check how the rows work
		if self.asciistart_right~=nil then		--check if the characters start to the right; if true then..
		
			start_row=self.rows-1			--then start at the right
			add_row=-1					--and move to left every time
		
		else					--they start to the left
		
			start_row=0		--then start at the left
			add_row=1		--and move to the right every time
		
		end
		
		if self.rows==0 then		--the fontsheet only goes up and down or down and up
		
			start_row=0
			add_row=0
			
		end
		
		-- Check how the columns work
		if self.asciistart_bottom~=nil then		--check if the characters start at the bottom; if true then..
		
			start_column=self.columns-1			--then start at the bottom
			add_column=-1					--and move up every time
		
		else					--they start at the the top
		
			start_column=0		--then start at the top
			add_column=1		--and move down every time
		
		end
		
		if self.columns==0 then		--the fontsheet only goes left and right or right and left
		
			start_column=0
			add_column=0
			
		end
		
		current_column=start_column
		current_row=start_row
		
		for current_character=1,256,1 do		--find the values for all ASCII characters
		
			-- Find pixel starting and stopping positions
			local loopx=char_width*current_row+(self.box_borderw*current_row)+self.box_borderw					--start pixel for checking where the character starts
			local loopy=char_height*current_column+(self.box_borderh*current_column)+self.box_borderh
			local loopx_end=loopx+char_width-1			--end pixel for checking where the pixel ends
			local loopy_end=loopy+char_height-1
			
			
			local found_pixel=false		--tells if a pixel has been found
			local no_pixel=false					--used to skip loops if no pixel exists for the character
			local r,g,b,a = 0					--used for getPixel
			local retrect = {x=0,y=0,w=0,h=0}
			local retquad=false
			
			local setheight=0				--used for _charheights
			local setwidth=0				--used for _charwidths
			
			cc_counter=cc_counter+1
			
			
			-- FIND THE LEFT SIDE
			for tempx=0,char_width-1,1 do				--go up and down moving to the right until a pixel is found
			
				for tempy=0,char_height-1,1 do
				
					r,g,b,a=fontsheet_data:getPixel(loopx+tempx,loopy+tempy)
					
					if a~=0 and found_pixel==false then		--a pixel has been found!
					
						retrect.x=loopx+tempx
						found_pixel=true
						--fontsheet_data:setPixel(loopx+tempx,loopy+tempy,255,0,0,255)			--RED
						break
					
					end
				
				end
				if found_pixel then break end
			
			end
			if found_pixel==false then no_pixel=true end
			found_pixel=false
			
			
			if no_pixel==false then			--if a pixel exists then continue, if not stop checking
			
				-- FIND THE RIGHT SIDE
				for tempx_end=0,char_width-1,1 do		--same as left side except go to the right
			
					for tempy=0,char_height-1,1 do
				
						r,g,b,a=fontsheet_data:getPixel(loopx_end-tempx_end,loopy+tempy)
				
						if a~=0 and found_pixel==false then		--a pixel has been found!
					
							retrect.w=loopx_end-tempx_end-retrect.x+1
							found_pixel=true
							setwidth=retrect.w
							--fontsheet_data:setPixel(loopx_end-tempx_end,loopy+tempy,0,255,0,255)			--GREEN
							break
					
						end
				
					end
					if found_pixel then break end
				
				end
				found_pixel=false
			
			
				-- FIND THE TOP
				for tempy=0,char_height-1,1 do		--go left and right moving down until a pixel is found
				
					for tempx=0,char_width-1,1 do
					
						r,g,b,a=fontsheet_data:getPixel(loopx+tempx,loopy+tempy)
				
						if a~=0 and found_pixel==false then		--a pixel has been found!
					
							retrect.y=loopy+tempy
							found_pixel=true
							setheight=tempy
							fontsheet_data:setPixel(loopx+tempx,loopy+tempy,0,0,255,255)			--BLUE
							break
					
						end
					
					end
					
				end
				found_pixel=false
				
				
				-- FIND THE BOTTOM
				for tempy_end=0,char_height-1,1 do		--same as top except move up
				
					for tempx=0,char_width-1,1 do
					
						r,g,b,a=fontsheet_data:getPixel(loopx+tempx,loopy_end-tempy_end)
				
						if a~=0 and found_pixel==false then		--a pixel has been found!
					
							retrect.h=loopy_end-tempy_end-retrect.y+1
							found_pixel=true
							fontsheet_data:setPixel(loopx+tempx,loopy_end-tempy_end,255,255,0,255)			--YELLOW
							break
					
						end
					
					end
				
				end
			
			end
			
			
			table.insert(self._charheights,current_character,setheight)
			table.insert(self._charwidths,current_character,setwidth)
			
			if self.upDown_sheet~=nil then		--if the fontsheet goes up and down then
			
				current_column=current_column+add_column			--go to the next column
				if(cc_counter>=self.columns) then					--column is at the last column
				
					cc_counter=0
					current_column=start_column					--restart the column
					current_row=current_row+add_row			--go to the next row
				
				end
				
			else			--if the fontsheet goes left and right
			
				current_row=current_row+add_row			--go to the next row
				if(cc_counter>=self.rows) then			--row is at the last row
				
					cc_counter=0
					current_row=start_row					--restart the row
					current_column=current_column+add_column		--go to the next column
				
				end
			
			end
			
			if no_pixel==false then retquad=love.graphics.newQuad(retrect.x,retrect.y,retrect.w,retrect.h,self.fontsheet:getDimensions())
			else retquad=0 end
			table.insert(self._charpositions,current_character,retquad)
		
		end	--end of loop
		
		if self.box_borderw==0 then self.box_borderw=nil end			--destroy border variables if not used
		if self.box_borderh==0 then self.box_borderh=nil end
		
		return true			--everything ran good
	
	else return false end			--returns that the problem was the fontsheet
	
end



function ASCII_font:draw(text, x, y, sx, sy)
	
	sx=sx or 1
	sy=sy or sx
	local start_x=x

	local ascii=0

	for i=1,string.len(text),1 do		--loop through each character of the string
	
		ascii=string.byte(text,i)			--convert the current character to ascii number
		
		if ascii==10 then 		--check if /n (new-line) was used; if so
		
			y=y+(self.vspacing*sy)			--move the placement down
			x=start_x
			i=i+1					--skip to the next i position (since \n is 2 characters)
		
		elseif ascii==32 then		--check if " " (space char) was used; if so
		
			x=x+(self.spacechar_spacing*sx)		--move the placement because of spacing
		
		elseif ascii==9 then		--tab spacing check
		
			x=x+((self.spacechar_spacing*sx)*self.tab_spacing)
		
		elseif self._charpositions[ascii+1]~=0 then			--draw the character if it exists
		
			love.graphics.draw(self.fontsheet,self._charpositions[ascii+1],x,y+(self._charheights[ascii+1]*sy),0,sx,sy,0,0,0,0)		--draw the fontsheet displaying only the character
			x=x+((self.char_spacing+self._charwidths[ascii+1])*sx)			--add spacing for the next character
				
		end
	
	end

end



function ASCII_font:drawChar(char, x, y, sx, sy)				--this will draw the character (it will not check for spaces, newline, etc.), this can accept full text

	sx=sx or 1
	sy=sy or sx
	local start_x=x

	local ascii=0
	
	for i=1,string.len(text),1 do
	
		ascii=string.byte(char)
		if self._charpositions[ascii+1]~=0 then love.graphics.draw(self.fontsheet,self._charpositions[ascii+1],x,y+self._charheights[ascii+1],0,sx,sy,0,0,0,0) end
		x=x+((self.char_spacing+self._charwidths[ascii+1])*sx)			--add spacing for the next character
		
	end

end



function ASCII_font:check(char)			--check if any pixels for the character exist

	local ascii=string.byte(char)
	if self._charpositions[ascii+1]==0 then return false			--the character doesn't exist
	else return true end							--the character exists!

end



function ASCII_font:destroy()

	self.fontsheet=nil
	self.rows=nil
	self.columns=nil
	self.box_borderw=nil
	self.box_borderh=nil
	self.spacechar_spacing=nil
	self.vspacing=nil
	self.tab_spacing=nil
	
	
	for i=1,table.getn(self._charpositions),1 do		--delete _charpositions
	
		self._charpositions[i]=nil
	
	end
	self._charpositions=nil
	
	
	for i=1,table.getn(self._charheights),1 do			--delete _charheights
	
		self._charheights[i]=nil
	
	end
	self._charheights=nil
	
	
	for i=1,table.getn(self._charwidths),1 do			--delete _charwidths
	
		self._charwidths[i]=nil
	
	end
	self._charwidths=nil
	
	asciistart_right=nil
	asciistart_bottom=nil
	upDown_sheet=nil
	

end
