control={					--if _ is set for control name(not the variable self.name) then it will not show when you try to edit controls; also - allows every player to have control, but it doesn't display the control when editing controls

	name="",
	input={},			--can store multiple inputs; stores them as strings
	state="none"		--"none", "pressed", "released", "down"; pressed switches to down, when released switches to released then none

}
control.__index=control
--for hats "h1zla"			--h marks it as hat, 1 is hat number, z marks the end of hat number, l marks the actual control, and a marks that "lu" and "ld" register the control; s would not allow "lu" and "ld" to register the control
--for axis "a1z+0.5"		--a marks it as axis; 1 is the axis number, z makes end of number, + marks that it is in positive axis(- for negative), 0.5 marks the dead-zone;NOTE: if you want true axis then use axis class



function control:new(variables)

	variables=variables or {}
	local self=setmetatable(variables,self)
	return self

end



function control:getInputData(input_number)				--gets the input properties;; returns input_type, input; can return hat_number, specific_direction

	input_number=input_number or 1
	if self.input[input_number]==nil then return false end		--control not found
	
	local retTable={}
	local ascii=string.byte(self.input[input_number],1,1)
	
	if ascii==107 then				--"k"; from keyboard
	
		retTable.input_type="keyboard"
		retTable.input=string.sub(self.input[input_number],2,string.len(self.input[input_number]))
	
	elseif ascii==106 then		--"j"; from joystick
	
		retTable.input_type="joystick"
		retTable.input=string.sub(self.input[input_number],2,string.len(self.input[input_number]))
	
	elseif ascii==104 then		--"h"; from joystick hat; also returns hat_number,input,specific_direction
	
		local actual_control=string.sub(self.input[input_number],2,string.len(self.input[input_number]))
		local hat_end=string.find(actual_control,"z")		--z marks the end of the hat number
	
		retTable.input_type="hat"
		retTable.hat_number=tonumber(string.sub(actual_control,1,hat_end-1))
		retTable.input=string.sub(actual_control,hat_end+1,string.len(actual_control)-1)
		retTable.specific_direction=false
		if string.sub(actual_control,string.len(actual_control),string.len(actual_control))=="s" then retTable.specific_direction=true end
	
	elseif ascii==109 then		--"m"; from mouse
	
		retTable.input_type="mouse"
		retTable.input=string.sub(self.input[input_number],2,string.len(self.input[input_number]))
		
	elseif ascii==97 then				--"a"; from axis; also returns axis_pos, deadzone, and axis_value;; does NOT return input
	
		local actual_control=string.sub(self.input[input_number],2,string.len(self.input[input_number]))
		local axis_end=string.find(actual_control,"z")
		
		retTable.input_type="axis"
		retTable.axis_num=string.sub(actual_control,1,axis_end-1)
		retTable.axis_pos=true
		if string.sub(actual_control,axis_end+1,axis_end+1)=="-" then retTable.axis_pos=false end
		retTable.deadzone=tonumber(string.sub(actual_control,axis_end+2,string.len(actual_control)))
			
		retTable.axis_value=joystick:getAxis(axis_num)			--the actual value of the axis on the joystick
	
	
	else return "type" end			--returns that the input type was not found
	
	return retTable

end



function control:update(joystick)

	local got_input=false
	local ascii
	local actual_control
	for i=1,table.getn(self.input),1 do
		
		ascii=string.byte(self.input[i],1,1)		--get the first char of the control; tells if from keyboard, joystick, etc.
		actual_control=string.sub(self.input[i],2,string.len(self.input[i]))		--get the rest of the string(is the actual control)
		if ascii==107 then				--"k"; input is from keyboard
			
			if got_input~=true then got_input=love.keyboard.isDown(actual_control) end
			
		elseif ascii==106 then		--"j"; input is from joystick
		
			if got_input~=true then got_input=player[i].joystick:isDown(actual_control) end
			
		elseif ascii==104 then			--"h"; input is from joystick hat
		
			local hat_end=string.find(actual_control,"z")		--z marks the end of the hat number
			local hat_num=tonumber(string.sub(actual_control,1,hat_end-1))
					
			local specific_direction=false
			if string.sub(actual_control,string.len(actual_control),string.len(actual_control))=="s" then specific_direction=true end
			
			actual_control=string.sub(actual_control,hat_end+1,string.len(actual_control)-1)
			if got_input~=nil then
				
				if actual_control==joystick:getHat(hat_num) then got_input=true end			--used for "ld", "lu", "ru", "rd"
							
				if specific_direction==false and string.len(actual_control)==1 then
				
					for z=1,string.len(actual_control),1 do		--uses for loop since "l" can have "lu" and "ld" also
				
						if actual_control==string.sub(joystick:getHat(hat_num),z,z) then
						
							got_input=true
							break					--get out of the loop
						
						end
				
					end
					
				end
				
			end
		
		elseif ascii==109 then		--"m"; input is from mouse
		
			if got_input~=true then got_input=love.mouse.isDown(actual_control) end
		
		elseif ascii==97 then		--"a"; input is from axis
		
			local axis_end=string.find(actual_control,"z")			--z makes the end of axis number
			local axis_num=tonumber(string.sub(actual_control,1,axis_end-1))
		
			local axis_pos=true
			if string.sub(actual_control,axis_end+1,axis_end+1)=="-" then axis_pos=false end
			local deadzone=tonumber(string.sub(actual_control,axis_end+2,string.len(actual_control)))
			print(joystick)
			local axis_value=joystick:getAxis(axis_num)			--the actual value of the axis on the joystick
			
			if (axis_pos==true and axis_value>deadzone) or (axis_pos==false and axis_value<-deadzone) then		--take the deadzone into account
			
				if got_input~=true then got_input=true end
			
			end
		
		end
			
	end

	if got_input==true then
	
		if self.state=="pressed" or self.state=="down" then self.state="down"			--switch from pressed to active
		else self.state="pressed" end
		
	else
	
		if self.state=="released" or self.state=="none" then self.state="none"				--switch from released to none
		else self.state="released" end
	
	end

end