custom_console =
{

	font=nil,					--uses ASCII font
	_maxInput=250,		--max amount of text input
	text={},					--stores multiple text lines
	time={}					--stores the time the text was received
	
}
custom_console.__index = custom_console



function custom_console:new(variables)

	variables=variables or {}
	local self=setmetatable(variables,self)
	return self

end



function custom_console:add(string,custom_time)

	custom_time=custom_time or love.timer.getTime()
	local pos=table.getn(self.text)+1
	local add_string={}			--actual string added; string is broken up for \n, newlines
	local prev_char=nil				--used to determine whether to put a blank line; used for newlines
	local cur_char=nil
	local cut_start=1					--where to start for the next line
	
	for i=1,string.len(string),1 do
	
		cur_char=string.byte(string,i,i)
		if prev_char==10 and cur_char==10 then			--2 newlines
		
			table.insert(add_string,table.getn(add_string)+1,string.sub(string,cut_start,i-2))		--break up the string
			--minus 2 is used since i and i-1 are newlines
			table.insert(add_string,table.getn(add_string)+1,"")	--create an empty line
			cut_start=i+1			--set the new pos
			
		end
		prev_char=cur_char
	
	end
	if table.getn(add_string)==0 then add_string[1]=string end			--no breaks within string
	
	for i=1,table.getn(add_string),1 do
	
		table.insert(self.text,pos,string)
		table.insert(self.time,pos,custom_time)
		if table.getn(self.text)>=self._maxInput+1 then	--remove last text if too many exist

			table.remove(self.text,1)			--remove first line of text
			
		end
		pos=table.getn(self.text)+1
	
	end
	
end



function custom_console:clear()

	for i=1,table.getn(self.text),1 do
	
		self.text[i]=nil
	
	end
	collectgarbage()

end



function custom_console:fitString(lines,w,start)		--returns a string that fits within area

	local current_lines=0
	local wrap=0					--amount of lines wrapped, used to remove current line(too big)
	local wrap_width=0		--not used
	local start=start+1 or table.getn(self.text)+1				--plus 1 used for i
	local tempstring={}			--stores last lines to first lines
	for i=1,lines,1 do
	
		if start-i<1 then break end			--exit loop if no more lines exist
		wrap_width,wrap=love.graphics.getFont():getWrap(self.text[start-i],w)
		current_lines=current_lines+wrap
		if current_lines+wrap>lines then			--there are more lines than allowed
		
			current_lines=current_lines-wrap			--remove wrap
			break
		
		end
		table.insert(tempstring,i,self.text[start-i])
		
	end
	
	local retstring=""
	local i=table.getn(tempstring)
	--_print(i)
	while i>0 do			--rearrange strings, string becomes backwards
	
		retstring=retstring .. tempstring[i] .. "\n"
		i=i-1
	
	end
	
	return retstring, current_lines			--returns the string and the amount of lines that were actually used
	
end



function custom_console:destroy()

	self._maxInput=nil
	for i=1,table.getn(self.text),1 do
	
		self.text[i]=nil
	
	end

end