entity_class = 
{

	spritesheet=nil,			--image with sprites
	sprite_quads = {},		--sprite positions
	boundingBox_draw = 			--used to check whether or not the entity should be drawn
	{
	
		x=0,
		y=0,
		w=0,
		h=0
		
	},
	script = {					--contains lua scripts
	
		create=nil,
		update=nil,
		collision=nil,
		draw=nil,
		destroy=nil
		
	}
	--special=true				--only used if the collision should be handled by the other object
	--canpush=nil			--only used if set true
	
}
entity_class.__index = entity_class



function entity_class:new(variables)

	variables=variables or {}
	variables._boundingBox_collision =	{}		--used for determining what direction the entities collided at
	
	variables.collision_rects = 					--stores multiple collision rectangles
	{
	
		--rectangles={}				--stores multiple rectangles (rectangles is a table which includes rectangles that are used for ONE COLLISION rectangle)
	
	}
	
	variables.boundingBox_draw=variables.boundingBox_draw or {}
	variables.script=variables.script or {}
	
	local self=setmetatable(variables,self)
	return self

end



function entity_class:createSpriteQuads(rectangles)

	if type(self.spritesheet)~=nil then

		local img_dimensions=self.spritesheet:getDimensions()
		for i=1,table.getn(rectangles),1 do
	
			self.sprite_quads[i]=love.graphics.newQuad(rectangles[i].x,rectangles[i].y,rectangles[i].w,rectangles[i].h,img_dimensions)

		end
		
	end

end



function entity_class:createCollisionRect(id,rectangles)

	self.collision_rects[id] = {}
	self.collision_rects[id].rectangles = {}
	
	for i=1,table.getn(rectangles),1 do
	
		self.collision_rects[id].rectangles[i] = {}
		self.collision_rects[id].rectangles[i].x=rectangles[i].x
		self.collision_rects[id].rectangles[i].y=rectangles[i].y
		self.collision_rects[id].rectangles[i].w=rectangles[i].w
		self.collision_rects[id].rectangles[i].h=rectangles[i].h
	
	end
	
	local minx,miny=rectangles[1].x,rectangles[1].y
	local maxx,maxy=rectangles[1].x+rectangles[1].w,rectangles[1].y+rectangles[1].h		--stores this way for rectangles that may be shifted
	
	for i=1,table.getn(rectangles),1 do				--gets a rectangle that contains all rectangles within it
	
		minx=math.min(minx,rectangles[i].x)
		miny=math.min(miny,rectangles[i].y)
		maxx=math.max(maxx,rectangles[i].x+rectangles[i].w)
		maxy=math.max(maxy,rectangles[i].y+rectangles[i].h)
	
	end
	
	self._boundingBox_collision[id]={			--set the boundingBox to the rectangle found before
	
		x=minx,
		y=miny,
		w=maxx-minx,
		h=maxy-miny
	
	}

end



function entity_class:removeCollisionRects(id)

	for i=1,table.getn(self.collision_rects[id].rectangles),1 do
	
		self.collision_rects[id].rectangles[i]=nil
	
	end

end



function entity_class:destroy()

	if pcall(self.class.script.destroy,self,"class") then return true		--run class destroy script
	else return false end

	self.spritesheet=nil
	self.script.create=nil
	self.script.update=nil
	self.script.collision=nil
	self.script.draw=nil
	self.script.destroy=nil
	self.canpush=nil
	
	for i=1,table.getn(self.sprite_quads),1 do		--delete sprite_quads
				
			table.remove(self.sprite_quads,i)
				
	end
	
	
	for i=1,table.getn(self.collision_rects),1 do		--delete collision_rects
	
		self:removeCollisionRects(i)
	
	end
	
	
	self.sprite_quads=nil
	
	collectgarbage()

end