entity_handler =
{

	freset=nil		--determines when to destroy self._free numbers

}
entity_handler.__index = entity_handler

function entity_handler:new(variables)

	variables=variables or {}
	variables._reserved = {}
	variables._entities = {}
	variables._free = {}
	local self=setmetatable(variables,self)
	return self

end



--ID Handling

function entity_handler:add(entity,custom_id)		--return true if successful, false if not

	local setid=1
	
	if table.getn(self._entities)>0 then				--check if their is atleast one entity
	
		if table.getn(self._free)==0 and custom_id==nil then 			--check if there any ids that have been freed
			
			local unique_id = true
			setid=table.getn(self._entities)+1
			
			while(true) do
			
				unique_id=true
				for i=1,table.getn(self._entities),1 do	--check if setid is in use
				
					if setid==self._entities[i].id then unique_id=false end		--id is already in use
				
				end
				
				if table.getn(self._reserved)~=0 then
				
					for i=1,table.getn(self._reserved),1 do		--check if setid is already used
					
						if setid==self._reserved[i] then unique_id=false end	--id is already used
					
					end
					
				end
				
				if unique_id==true then break		--id not in used
				else setid=setid+1 end				--change setid and try again(loop)
			
			end
			
		elseif custom_id==nil then		--ids are freed and check if a custom id is set
		
			setid=self._free[table.getn(self._free)]			--use the first (since its the oldest)
			table.remove(self._free,table.getn(self._free))			--remove the id from list
		
		else					--a custom id is set
		
			for i=1,table.getn(self._entities),1 do		--check if the id is used
	
				if custom_id==self._entities[i].id then return false end	--unable to add entity (already in use)
				setid=custom_id
	
			end
			
		end
	
	else
	
		if table.getn(self._free)~=0 and custom_id==nil then 			--check if there any ids that have been freed
		
			--there are freed ids
			setid=self._free[1]			--use the first (since its the oldest)
			table.remove(self._free,1)			--remove the id from list
		
		elseif custom_id~=nil then setid=custom_id					--a custom id is set
		end
	
	end
	
	entity.id=setid
	table.insert(self._entities,table.getn(self._entities)+1,entity)
	return true							--entity successfully added

end



function entity_handler:free(id, store_custom_ids)

	store_custom_ids=store_custom_ids or false
	
	for i=1,table.getn(self._entities),1 do
	
		if id==self._entities[i].id then
		
			if type(id)~="number" and store_custom_ids==false then		--check if the id is number
			else table.insert(self._free,table.getn(self._free)+1,id) end
			
			self._entities[i]:destroy()				--run entity destroy function
			table.remove(self._entities,i)

			if table.getn(self._free)>=self.freset then		--check if _free is at limit
			
				for i=table.getn(self._entities),1,1 do		--destroy _free table
				
					table.remove(self._entities,i)			--destroy each var one by one
				
				end
				
				self._free=nil
				collectgarbage()					--remove _free vars from memory
				
				self._free = {}
				
			end
			
			return true		--entity successfully freed
			
		end
		
	end
	
	return false		--entity id not found

end



function entity_handler:reserve(id)		--does not allow an id to be used be conventional means

	for i=1,table.getn(self._entities),1	do	--check if the id is used
	
		if self._entities[i].id==id then return false end	--unable to reserve id (already in use)
	
	end

	table.insert(self._reserved,table.getn(self._reserved)+1,id)		--reserve the id
	return true							--the id was successfully reserved

end



function entity_handler:reserveForce(id)	--same as id but will change other entity id if in use

	for i=1,table.getn(self._entities),1 do		--check if the id is used
	
		if self._entities[i].id==id then		--id in use so change the id of entity
		
			local newid=table.getn(self._entities)
			
			while(true) do
			
				if self:changeId(i,newid)==2 then break
				else newid=newid+1 end
				
			end
		
		end	
	
	end
	
end



function entity_handler:release(id,free)

	free=free or true	--true=add id to self._free
	for i=1,table.getn(self._reserved),1 do
	
		if id==self._reserved[i] then
		
			table.remove(self._reserved,i)
			if free==true then table.insert(self._free,table.getn(self._free),id) end
			return true		--id successfully released
		
		end
	
	end

	return false		--id not reserved

end



function entity_handler:changeId(id, newid)

	local success="not found"
	
	for i=1,table.getn(self._entities),1 do		--check if the id is used
	
		if self._entities[i].id==id then	--id found
		
			success="in use"
			break
		
		end
	
	end
	
	if success==1 then
	
		success="success"
		for i=1,table.getn(self._entities),1 do
	
			if self._entities[i].id==newid then		--newid in use
			
				success="in use"
				break
				
			end
	
		end
	
	end
	
	return success

end



function entity_handler:checkId(id)		--get id details

	for i=1,table.getn(self._entities),1 do
	
		if id==self._entities[i].id then return "in use" end	--id is in use
	
	end
	
	for i=1,table.getn(self._reserved),1 do
	
		if id==self._reserved[i] then return "reserved" end	--id is reserved
	
	end
	
	for i=1,table.getn(self._free),1 do
	
		if id==self._free[i] then return "free" end		--id is free
	
	end
	
	return "not in use" 		--not in use

end



function entity_handler:returnIds()

	local used_ids = {}

	for i=1,table.getn(self._entities),1 do
	
		used_ids[i]=self._entities[i].id
	
	end
	
	return used_ids

end



function entity_handler:getAmount()

	return table.getn(self._entities)	--get the amount of entities
	
end



function entity_handler:returnEntity(id)

	for i=1,table.getn(self._entities),1 do
	
		if self._entities[i].id==id then return self._entities[i] end
	
	end

	return false		--entity with id does not exist

end





--Game Handling

function entity_handler:runScript(script_name,...) --"create", "update", "draw", "collision", "destroy"

	if script_name=="draw" then love.graphics.setColor(255,255,255,255) end
	for i=1,table.getn(self._entities),1 do
	
		self._entities[i]:runScript(script_name,...)
	
	end

end



function entity_handler:runFunction(func)		--runs function for every entity

	func=func or nil
	if func==nil then return false end				--no function found
	for i=1,table.getn(self._entities),1 do
	
		pcall(func,self._entities[i])
	
	end

end



function entity_handler:destroy()

	self.freset=nil

	for i=1,table.getn(self._entities),1 do			--delete entities
				
			self._entities[i]:destroy()
			table.remove(self._entities,i)
				
	end
	
	for i=1,table.getn(self._reserved),1 do			--delete reserved id list
				
			table.remove(self._reserved,i)
				
	end
	
	for i=1,table.getn(self._free),1 do				--delete free id list
				
			table.remove(self._free,i)
				
	end
	
	self._entities=nil
	self._reserved=nil
	self._free=nil
	self.freset=nil
	
	collectgarbage()				--remove vars from memory

end
